import 'dart:io';
import 'dart:ui';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/widgets/wow_cache_manager/wow_cache_manager.dart';

class WowImage<T> extends StatefulWidget {
  const WowImage(
    this.dataImage, {
    super.key,
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
    this.wImageLoadding,
    this.hImageLoadding,
    this.isCached = false,
  }) : _borderRadius = null;

  const WowImage.border(
    this.dataImage, {
    super.key,
    this.fieldName,
    this.height,
    this.width,
    this.errorBuilder,
    this.boxFit = BoxFit.cover,
    this.loaddingBuilder,
    this.scale = 1.0,
    this.wImageLoadding,
    this.hImageLoadding,
    BorderRadius? borderRadius,
    this.isCached = false,
  }) : _borderRadius = borderRadius;

  /// Data hình ảnh
  //final Map<String, Object?> dataImage;
  final T dataImage;

  /// field name hình ảnh [fieldName]!= ['hinhAnh']
  final String? fieldName;

  final BorderRadius? _borderRadius;

  final double? width;
  final double? height;

  final Widget? errorBuilder;

  final BoxFit boxFit;

  final Widget? loaddingBuilder;

  final double scale;

  final double? wImageLoadding, hImageLoadding;

  final bool isCached;

  @override
  State<WowImage> createState() => _WowImageState<T>();
}

class _WowImageState<T> extends State<WowImage> {
  late T dataImage;

  final Map<String, Object?> data = {};

  String get pathImage => widget.fieldName != null
      ? widget.fieldName! + '_image_path'
      : 'hinhAnh_image_path';

  String get pathSmall => widget.fieldName != null
      ? widget.fieldName! + '_small_path'
      : 'hinhAnh_small_path';

  String get thumbnailUrl => Methods.getString(data, pathSmall);
  String get imageUrl => Methods.getString(data, pathImage);

  @override
  void initState() {
    _loadImage();
    super.initState();
  }

  @override
  void didUpdateWidget(covariant WowImage oldWidget) {
    bool check = false;
    if (dataImage != widget.dataImage) {
      check = true;
    }
    if (widget.dataImage is Map<String, Object>) {
      String newImagePath = Methods.getString(widget.dataImage, pathImage);
      String oldImagePath = imageUrl;
      if (newImagePath == oldImagePath) {
        check = false;
      } else {
        check = true;
      }
    }
    if (check) {
      setState(
        () {
          _loadImage();
        },
      );
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    if (dataImage is File) {
      return _boderImage(
        _fileImage(Methods.getListFile(data).first),
      );
    }
    return _boderImage(
      _cacheImage(),
    );
  }

  Widget _boderImage(Widget child) {
    if (widget._borderRadius == null) {
      return child;
    } else {
      return ClipRRect(
        borderRadius: widget._borderRadius,
        child: child,
      );
    }
  }

  // cache hình ảnh gốc
  CachedNetworkImage _cacheImage() {
    return CachedNetworkImage(
      imageUrl: Methods.getURL(imageUrl),
      width: widget.width,
      height: widget.height,
      fit: widget.boxFit,
      errorWidget: (_, __, ___) {
        return widget.errorBuilder ?? errorImage();
      },
      progressIndicatorBuilder: (_, __, ___) {
        return widget.loaddingBuilder ??
            Stack(
              children: [
                Opacity(
                  opacity: 1.0,
                  child: BackdropFilter(
                    filter: ImageFilter.blur(sigmaX: 20, sigmaY: 20),
                    child: Container(
                      color: Colors.black.withOpacity(0),
                      child: Opacity(
                        opacity: 0.0,
                        child: SizedBox(
                          width: widget.wImageLoadding ??
                              widget.width ??
                              double.infinity,
                          height: widget.hImageLoadding ??
                              widget.height ??
                              double.infinity,
                        ),
                      ),
                    ),
                  ),
                ),
                _imageNetWork(
                  Methods.getURL(thumbnailUrl),
                )
              ],
            );
      },
    );
  }

  Image _fileImage(File file) {
    return Image(
      image: FileImage(file, scale: widget.scale),
      width: widget.width,
      height: widget.height,
      fit: widget.boxFit,
      errorBuilder: (_, __, ___) => widget.errorBuilder ?? errorImage(),
      frameBuilder: (_, child, frame, __) {
        if (frame != null) {
          return child;
        }
        return errorImage();
      },
    );
  }

  Image _imageNetWork(
    String src, {
    Widget? errorBuider,
    Color? color,
    BlendMode? blendMode,
    Widget Function(BuildContext, Widget, int?, bool)? frameBuilder,
  }) {
    return Image(
      image: NetworkImage(src, scale: widget.scale),
      color: color,
      gaplessPlayback: true,
      excludeFromSemantics: true,
      colorBlendMode: blendMode,
      errorBuilder: (context, error, stackTrace) => errorBuider ?? errorImage(),
      width: widget.wImageLoadding ?? widget.width ?? double.infinity,
      height: widget.hImageLoadding ?? widget.height ?? double.infinity,
      fit: widget.boxFit,
      frameBuilder: frameBuilder ??
          (context, child, frame, wasSynchronouslyLoaded) {
            if (frame != null) {
              return child;
            }
            return errorImage();
          },
    );
  }

  Image errorImage({Color? color, BlendMode? blendMode}) {
    return Image.asset(
      'assets/images/default.jpg',
      width: widget.width,
      height: widget.height,
      fit: widget.boxFit,
    );
  }

// load hình ảnh
  void _loadImage() {
    dataImage = widget.dataImage;
    data.clear();
    if (dataImage is Map<String, Object?>) {
      data.addAll(dataImage as Map<String, Object?>);
    } else if (dataImage is String) {
      data[pathImage] = dataImage;
    } else if (widget.dataImage is File) {
      data['file'] = dataImage;
    }
    if (widget.isCached) {
      WowCacheManager.getImage(Methods.getUrlByData(data, pathImage)).then(
        (file) {
          if (mounted) {
            setState(() {
              data['file'] = file;
            });
          }
        },
      );
    }
  }
}
